;;;; cl-scopes/test-csys - testing for the scopes-csys system.

(defpackage :scopes/test-csys
  (:use :common-lisp)
  (:local-nicknames (:alx :alexandria)
                    (:actor :scopes/core/actor)
                    (:async :scopes/util/async)
                    (:config :scopes/config)
                    (:core :scopes/core)
                    (:csys :scopes/csys)
                    (:logging :scopes/logging)
                    (:message :scopes/core/message)
                    (:shape :scopes/shape)
                    (:util :scopes/util)
                    (:t :scopes/testing))
  (:export #:run)
  (:import-from :scopes/testing #:deftest #:== #:!=))

(in-package :scopes/test-csys)

;;;; testing environment

(defclass test-env (csys:environment)
  ((test-suite :reader test-suite :initarg :test-suite)))

(defun probe (msg state syns env)
  (let ((t:*test-suite* (test-suite env)))
    (== (shape:data msg) (pop state))
    (actor:become (csys:neuron #'probe state syns env))))

(defvar *probe* nil)

;;;; test runner

(defun run ()
  (async:init)
  (let* ((t:*test-suite* (make-instance 't:test-suite :name "csys"))
         (csys:*sensors* (make-hash-table :test #'equal))
         (csys:*environment* (make-instance 'test-env :test-suite t:*test-suite*))
         (*probe* (actor:create 
                    (csys:neuron #'probe '(1 3 4 5) nil csys:*environment*))))
    (load (t:test-path "config-csys" "etc"))
    (core:setup-services)
    (unwind-protect
      (progn
        (test-basic))
      (sleep 0.1)
      (async:finish)
      (t:show-result))))

(deftest test-basic ()
  (csys:send-message '(:csys :add :log :s1) 1)
  (csys:send-message '(:csys :add :log :s1) 3)
  (csys:send-message '(:csys :sub :log :s2) 4)
  (csys:send-message '(:csys :add :log :s2) 5)
)
